﻿----------------------------------------
-- Epic enchant List v 0.1, Copyright 2010 Wolfgang Rufeger, wolfgang@rufeger.de
-- 0.1	Initial version
-- 0.2	completed Cataclysm, added some WotLK
-- 0.3	most enchants BC-CATA (Classic still missing)
-- 0.4  changed toc fpr 4.3.0
-- 0.5  fixed WotLK leg/feet enchants
----------------------------------------


-- GLOBAL VARIABLES ------------------------------------------------
EEL_TOPVER = "0";
EEL_SUBVER = "5";

EEL_VER = EEL_TOPVER.."."..EEL_SUBVER;
EEL_VERSION = "ver "..EEL_VER.." by Ydy @ Shattrath"

EEL_SLOTSELECT = _G["ITEMSLOTTEXT"];
EEL_STATSELECT = _G["STATS_LABEL"];
EEL_CLOSE = _G["CLOSE"];
EEL_JC = EEL_LocaleText["jc"];

EEL_SCAN_FRAME_TITLE = "Epic Enchant List";
EEL_SCAN_FRAME_SUBTITLE = "Retrieving Enchant Data from server ...";
EEL_SCAN_FRAME_ABORT = "cancel";
EEL_SCAN_FRAME_INITIALIZING = "Initializing scan ...";
EEL_QUALITY = _G["QUALITY"];
EEL_ITEMLEVEL = _G["LEVEL"];

EEL_SHOWFAVORITES = EEL_LocaleText["fav"];
EEL_HIDEFAVORITES = _G["ALL"];  
EELBUILD = nil;
EELDBVER = nil;
eel_rs = nil;
EEL_DB_LOADED = nil;
tscurrent = 0;
tsmax = 0;

-- LOCAL VARIABLES ------------------------------------------------
local retries = 0;
local failed = 0;
local waitTable = {};
local waitFrame = nil;
local MY_ADDON_NAME = "EpicEnchantList";
local MY_ADDON_VERSION = EEL_TOPVER..EEL_SUBVER;
local EEL_Scanned_Enchants = 0;
-- INITIALIZE UNSET VARS ------------------------------------------------

if(EEL_Options == nil) then  -- saving options per char
    EEL_Options = {};
end

if(EELDB == nil) then -- saving gem data per account
    EELDB = {};
end

if (EEL_FAV == nil) then -- saving favorites per char
	EEL_FAV = {};
end

if ( EEL_Options["SelectedStat"]  == nil ) then
    EEL_Options["SelectedStat"] = 0;
end

if ( EEL_Options["SelectedSlot"] == nil ) then
    EEL_Options["SelectedSlot"] = 0;
end

if ( EEL_Options["SelectedLevelOp"] == nil ) then
    EEL_Options["SelectedLevelOp"] = 1;
end

if ( EEL_Options["SelectedLevel"] == nil ) then
    EEL_Options["SelectedLevel"] = 2;
end

if ( EEL_Options["buy"]  == nil ) then
    EEL_Options["buy"] = false;
end

if ( EEL_Options["loot"]  == nil ) then
    EEL_Options["loot"] = false;
end

if ( EEL_Options["details"]  == nil ) then
    EEL_Options["details"] = false;
end

if ( EEL_Options["favorites"] == nil ) then
	EEL_Options["favorites"] = false;
end

if ( EEL_Options["recommended"] == nil ) then
	EEL_Options["recommended"] = false;
end

if (EEL_Options["debug"] == nil) then
	EEL_Options["debug"] = false
end

if (EEL_Options["minimap"] == nil) then
	EEL_Options["minimap"] = true
end


-- MISC FUNCTIONS ------------------------------------------------

---- Chat Console Output ----
function out(text)
	DEFAULT_CHAT_FRAME:AddMessage(text)
	--UIErrorsFrame:AddMessage(text, 1.0, 1.0, 0, 1, 10) 
end

---- Delayed script execution ----
function EEL_wait(delay, func, ...)
  if(type(delay)~="number" or type(func)~="function") then
    return false;
  end
  if(waitFrame == nil) then
    waitFrame = CreateFrame("Frame","WaitFrame", UIParent);
    waitFrame:SetScript("onUpdate",function (self,elapse)
      local count = #waitTable;
      local i = 1;
      while(i<=count) do
        local waitRecord = tremove(waitTable,i);
        local d = tremove(waitRecord,1);
        local f = tremove(waitRecord,1);
        local p = tremove(waitRecord,1);
        if(d>elapse) then
          tinsert(waitTable,i,{d-elapse,f,p});
          i = i + 1;
        else
          count = count - 1;
          f(unpack(p));
        end
      end
    end);
  end
  tinsert(waitTable,{delay,func,{...}});
  return true;
end

--- counting table rows with every index format
function eel_tcount(tab)
   local n = #tab
   if (n == 0) then
     for _ in pairs(tab) do
       n = n + 1
     end
   end
   return n
end

--- Market Price from Auctioneer, if installed
function EEL_MarketPrice(itemID)
	if not itemID then
		if EEL_Options["debug"] then
			return "noItemID";
		end;
		return "";
	end
	aucinstalled = false;
	itemName, itemLink, itemQuality, itemLevel, itemTexture = EEL_EnchantInfo(itemID);
	if AucAdvanced then -- Auctioneer
		sellPrice, numSeen = AucAdvanced.API.GetMarketValue(itemLink);
		aucinstalled = true;
	-- elseif Atr_GetAuctionBuyout then -- Auctionator
		-- sellPrice = Atr_GetAuctionBuyout(itemID);
		-- aucinstalled = true;
	-- elseif AuctionLite then -- AuctionLite
		-- local hist = AuctionLite:GetHistoricalPriceById(itemID, 0);
		-- sellPrice = math.floor(hist.price);
		-- aucinstalled = true;
	-- elseif IsAddOnLoaded("AuctionMaster") and (AucMasStatisticVersion and AucMasStatisticVersion.major==1) then -- AuctionMaster
		-- avgBid, sellPrice, numBid, numBuy = AucMasGetAuctionInfo(itemLink, false);
		-- aucinstalled = true;	
	end;
	if sellPrice == nil then
		return "";
	else
		sellPrice = tonumber(sellPrice);
	end
	if aucinstalled then
		if (sellPrice ~= nil) and (sellPrice > 0) then
			local g = math.floor(sellPrice/10000);
			local GSC_GOLD = "ff807000"
			local digits = math.floor(math.log10(sellPrice)+1)
			if digits < 9 then
				gsc = string.format("|c%s%5d|r", GSC_GOLD, g)
			else
				gsc = string.format("|c%s%2.1fk|r", GSC_GOLD, g/1000)
			end
			if g == 0 then
				return "";
			end
			return gsc;
		end
	end
	return "";
end

--- scan 2. line of tooltip ---
function EEL_GetToolTipStats(itemid)
	ScanTooltip:ClearLines();
	ScanTooltip:SetOwner(WorldFrame, "ANCHOR_NONE");
    ScanTooltip:SetHyperlink("item:"..itemid..":0:0:0");
	itemStats = "";
	for i=2,ScanTooltip:NumLines() do
		local mytext=_G["ScanTooltipTextLeft"..i];
		local text=mytext:GetText();
		if i == 2 then
			itemStats = text; -- use first line on default
		end
		if string.find(text, "+") then
			itemStats = text; -- use line with + in text for stats
		end
 	end
	return itemStats;
end

--- LibVersionCheck ---

EELAddon = {}
local lvcLib = LibStub:GetLibrary( "LibVersionCheck-1.0" )
lvcLib:RegisterVersion( MY_ADDON_NAME, MY_ADDON_VERSION, EELAddon, "EELVersionCallback" )
local newestVersion = tonumber( strmatch( MY_ADDON_VERSION, "%d+" ) )

function EELAddon:EELVersionCallback( sender, identifier, version )
    if identifier == MY_ADDON_NAME and version then
        local thisVersion = tonumber( strmatch( version, "%d+" ) )
        if thisVersion > newestVersion then
            newestVersion = thisVersion
            DEFAULT_CHAT_FRAME:AddMessage( string.format( "%s has a newer version of %s. (%s)", sender, MY_ADDON_NAME, version ) )
			DEFAULT_CHAT_FRAME:AddMessage("Download from http://wow.curse.com/downloads/wow-addons/details/egl.aspx");
        end
    end
end




-- LDB BROKER ------------------------------------------------

local ldb = LibStub:GetLibrary("LibDataBroker-1.1",true)
ldb:NewDataObject("Epic Enchant List", {
    type = "launcher",
	label = "EpicEnchantList",
	OnClick = function(_, msg)
        if msg == "LeftButton" then
	        if EEL_Form:IsVisible() then
		        EEL_Form:Hide()
		    else
		        EEL_Form:Show()
	        end
        elseif msg == "RightButton" then
			if not EEL_Config:IsVisible() then
				EEL_Config:Show()
			else
				EEL_Config:Hide()
			end
		end
    end,
	icon = "Interface\\AddOns\\EpicEnchantList\\icon",
	OnTooltipShow = function(tooltip)
		if not tooltip or not tooltip.AddLine then return end
		tooltip:AddLine("Epic Enchant List")
	end,
})


-- SLASH HANDLER ------------------------------------------------

function EEL_SlashCommandHandler(msg)
	if (msg == "debug") then
		if EEL_Options["debug"] then
			EEL_Options["debug"] = false;
			out("EEL: Debugging disabled");
		else
			EEL_Options["debug"] = true;
			if (EEL_Form:IsVisible() == false) then
				EEL_Form:Show()
			end
			
			out("EEL: Debugging enabled");
		end
	elseif (msg == "cata") then
		if EEL_Options["cataclysm"] then
			EEL_Options["cataclysm"] = false;
			out("EEL: Cataclysm support disabled");
		else
			EEL_Options["cataclysm"] = true;
			out("EEL: Cataclysm support enabled");
		end
	elseif (msg == "minimap") then
		if EEL_MinimapButton:IsVisible() then
			EEL_Options["minimap"] = false;
			EEL_MinimapButton:Hide();
		else
			EEL_Options["minimap"] = true;
			EEL_MinimapButton:Show();
		end
	elseif (msg == "scan") then
		EEL_ForceScan();
		if (EEL_Form:IsVisible() == false) then
			EEL_Form:Show();
		end
	elseif (msg == "show") then
		EEL_Form:Show()
	elseif (msg == "hide") then
		EEL_Form:Hide()
	elseif (msg == "config") then
		if not EEL_Config:IsVisible() then
			EEL_Config:Show()
		else
			EEL_Config:Hide()
		end
	else
		out("|cFFFFFF00Epic Enchant List: ");
		out("|cFFFFFF00("..EEL_VERSION..")");
		out("|cFF00FF00/eel show |cFFFFFFFF[show main window]");
		out("|cFF00FF00/eel hide |cFFFFFFFF[hide main window]");
		out("|cFF00FF00/eel config |cFFFFFFFF[show config window]");
		out("|cFF00FF00/eel minimap |cFFFFFFFF[show/hide minimap button]");
		out("|cFF00FF00/eel debug |cFFFFFFFF[show/hide debugging messages]");
	end
end

-- ON LOAD ------------------------------------------------

function EEL_OnLoad(this)
	EEL_Form:RegisterEvent("VARIABLES_LOADED");
	eelf = _G["EEL_Form"];
	SLASH_EEL1 = "/epicenchantlist";
	SLASH_EEL2 = "/eel";
	SlashCmdList["EEL"] = function(msg)
		EEL_SlashCommandHandler(msg);
	end
	if EEL_Form:IsVisible() then
		EEL_Form:Hide()
	else
		EEL_Form:Show()
	end
	EEL_Options = {};
	EEL_Options["SelectedSlot"] = 0;
	EEL_Options["SelectedStat"] = 0;
	EEL_Options["SelectedLevelOp"] = 1;
	EEL_Options["SelectedLevel"] = 2;
	EEL_Options["debug"] = false;
	EEL_Options["details"] = false;
	EEL_Options["favorites"] = false;
	EEL_Options["cataclysm"] = true;
	local version, build, builddate, tocversion = GetBuildInfo();
end

-- ON SHOW ------------------------------------------------

function EEL_OnShow()
	if EEL_Options["details"] then
		CheckButtonDetails:SetChecked(true);
	else
		CheckButtonDetails:SetChecked(false);
	end	
	local favb = _G["EEL_FavButton"];
	if EEL_Options["favorites"] == true then
		favb:SetText(EEL_HIDEFAVORITES);
	else
		favb:SetText(EEL_SHOWFAVORITES);
	end	
	for i = 1, 15 do
		item = _G["EELItem_"..i];
		item:Hide();
	end
	EEL_ScrollBar_Update();
end

-- ON EVENT ------------------------------------------------

function EEL_OnEvent(event)
	if (event == "VARIABLES_LOADED") then
		local version, build, builddate, tocversion = GetBuildInfo();
		local mybuild = version..build;
		EEL_DB_LOADED = true;
		totalgems = 0;
		eel_cata = eel_tcount(EEL_Data_WotLK);
		eel_wotlk = eel_tcount(EEL_Data_WotLK);
		eel_bc = eel_tcount(EEL_Data_BC);
		eel_classic = eel_tcount(EEL_Data_Classic);
		totalenchants = eel_bc + eel_wotlk  + eel_cata + eel_classic;
		out("EpicEnchantList loaded ("..totalenchants.." enchants) ...");
		EEL_Options["cataclysm"] = true;
		--EELBUILD = nil; -- while DEVELOPING
		if (EELBUILD == nil) or (EELDBVER == nil) then
			EELBUILD = mybuild;
			EELDBVER = EEL_VER;
			out("EEL: Scanning enchants for local database ...");
			EEL_Scan();
		end
		if (EELBUILD ~= mybuild) or (EELDBVER < EEL_VER) then
			EELBUILD = mybuild;
			EELDBVER = EEL_VER;
			out("EEL: Scanning enchants for new WOW-Client "..version.." (Build "..build.." / "..builddate..") and EEL "..EEL_VER);
			EEL_Scan();
		end
		EEL_Options["cataclysm"] = true;
		if EEL_Options["minimap"] == true then
			EEL_MinimapButton:Show()
			CheckButtonConfig1:SetChecked(true);
		else
			EEL_MinimapButton:Hide()
			CheckButtonConfig1:SetChecked(false);
		end
		if EEL_Options["debug"] == true then
			CheckButtonConfig5:SetChecked(true);
		else
			CheckButtonConfig5:SetChecked(false);
		end
		EEL_SlotDropDown_Init();
		EEL_StatDropDown_Init();
		EEL_ScrollBar_Update();
	end
end


-- GET ENCHANT INFO ------------------------------------------------

function EEL_EnchantInfo(value)
	if EEL_DB_LOADED then
		if EELDB[value] ~= nil then 
			local iteminfo = EELDB[value];
			if iteminfo[6] == nil then
				iteminfo[6] = EEL_GetToolTipStats(value);
				EELDB[value] = { iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5], iteminfo[6] }
			end
			return iteminfo[1], iteminfo[2], iteminfo[3], iteminfo[4], iteminfo[5], iteminfo[6];
		else
			EEL_GetEnchantInfo(value);
			return false;
		end
	else
		if EEL_Options["debug"] then
			out("EEL DEBUG: EELDB not yet loaded");
		end
		return false;
	end
end
-- HELP BUTTON MOUSEOVER ------------------------------------------------

function EEL_Help_OnEnter(this)
	local entries = table.getn(EEL_Help);
	EELTooltip:ClearLines();
	EELTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EELTooltip:AddLine(EEL_Help[i]);
	end
    EELTooltip:Show();
end

function EEL_Help_OnLeave()
  EELTooltip:Hide();
end
-- FAV BUTTON ------------------------------------------------

function EEL_Fav_OnEnter(this)
	local entries = table.getn(EEL_Fav);
	EELTooltip:ClearLines();
	EELTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
	for i=0,entries do
		EELTooltip:AddLine(EEL_Fav[i]);
	end
    EELTooltip:Show();
end

function EEL_Fav_OnLeave()
  EELTooltip:Hide();
end

function EEL_Fav_OnClick(this)
	local favb = _G["EEL_FavButton"];
	local recb = _G["EEL_RecButton"];
	if EEL_Options["favorites"] == true then
		EEL_Options["favorites"] = false;
		favb:SetText(EEL_SHOWFAVORITES);
	else
		EEL_Options["favorites"] = true;
		favb:SetText(ALL);
	end
	EEL_ScrollBar_Update();
end


-- ITEM BUTTON FUNCTIONS ------------------------------------------------

function EELItem_OnEnter(this)
local addmembers = "GuildCrafters: ";


	EELTooltip:ClearLines();
	local id = this:GetID()+ FauxScrollFrame_GetOffset(EEL_ScrollBar);  
	if id ~= nil then
		local itemName, itemLink, itemQuality, itemLevel, itemTexture = EEL_EnchantInfo(EEL[id]);
		EELTooltip:SetOwner(this, "ANCHOR_RIGHT", -(this:GetWidth() / 2), 24);
		EELTooltip:SetHyperlink("item:"..EEL[id]..":0:0:0");
		EELTooltip:AddLine("Item Level: "..itemLevel..", ID: "..EEL[id], 0.7,0.7,1);
		EELTooltip:Show();
	end 
end

function EELItem_OnLeave()
  SELECTEDITEM = 0;
  EELTooltip:Hide();
end

function EELItem_OnClick(this)
	local EEL_Skill_ID;
	local id = this:GetID() + FauxScrollFrame_GetOffset(EEL_ScrollBar);
	local itemName, itemLink, itemQuality, itemLevel, itemTexture = EEL_EnchantInfo(EEL[id]);

	local editbox = ChatFrameEditBox
	if not editbox then
		editbox = ChatEdit_GetActiveWindow();
    end 
    if AuctionFrame then
        if (AuctionFrame:IsVisible() and CanSendAuctionQuery() and itemName) then
            BrowseName:SetText(itemName);
            AuctionFrameBrowse_Search();
        end
    end
	if (editbox and IsShiftKeyDown()) then
		editbox:Insert(itemLink); 
	end
	if IsControlKeyDown() then
		if EEL_FAV[EEL[id]] ~= nil then
			EEL_FAV[EEL[id]] = nil;
			EEL_ScrollBar_Update();
		else
			EEL_FAV[EEL[id]] = true;
			EEL_ScrollBar_Update();
		end
	end
end


-- RESULT LIST ------------------------------------------------



function EEL_HasNotStat(eeltable,value)
	if EEL_Options["SelectedStat"] ~= OOO and 
	 (eeltable[value]["stat1"] ~= EEL_Options["SelectedStat"] and eeltable[value]["stat2"] ~= EEL_Options["SelectedStat"] and eeltable[value]["stat3"] ~= EEL_Options["SelectedStat"]) then
		ret = true;
	else
		ret = false;
	end
	if EEL_Options["SelectedStat"] == STR or EEL_Options["SelectedStat"] == INT or EEL_Options["SelectedStat"] == SPI or EEL_Options["SelectedStat"] == AGI then
		if eeltable[value]["stat1"] == AAA or eeltable[value]["stat2"] == AAA or eeltable[value]["stat3"] == AAA  then
			ret = false;
		end
	end
	return ret;
end

function EEL_IsNotSlot(eeltable,value)
	if EEL_Options["SelectedSlot"] ~= OOO and 
	  EEL_Options["SelectedSlot"] ~= eeltable[value]["slot1"] and EEL_Options["SelectedSlot"] ~= eeltable[value]["slot2"] then
		return true;
	else
		return false;
	end
end


function EEL_ShowLevel(selected,required)
	if EEL_Options["SelectedLevelOp"] == 0 then
		if selected == required then
			return true;
		else
			return false;
		end
	elseif EEL_Options["SelectedLevelOp"] == 1 then
		if selected <= required then
			return true;
		else
			return false;
		end
	elseif EEL_Options["SelectedLevelOp"] == 2 then
		if selected >= required then
			return true;
		else
			return false;
		end
	end
	return false;
end


function EEL_ScrollBar_Update()
    local line, lineplusoffset, text, extratext;
    local currentgem;    
    EEL = {}; 
    currentenchant = 1;
	eel_shown_enchants = 0;
	
	if (EEL_ShowLevel(EEL_Options["SelectedLevel"],0)) then	-- ALL, Classic
		for value,_ in pairs(EEL_Data_Classic) do    
			additem = true;	    	    
			if EEL_Options["favorites"] then
				if EEL_FAV[value] == nil then
					additem = false;
				end
			end
			if EEL_HasNotStat(EEL_Data_Classic,value) then
				additem = false;
			elseif EEL_IsNotSlot(EEL_Data_Classic,value) then
				additem = false;
			elseif (EEL_EnchantInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EEL, value);
			end        
			currentenchant = currentenchant + 1;
		end      
	end


	if (EEL_ShowLevel(EEL_Options["SelectedLevel"],1)) then -- ALL, BC
		for value,_ in pairs(EEL_Data_BC) do    
			additem = true;	  
			if EEL_Options["favorites"] then
				if EEL_FAV[value] == nil then
					additem = false;
				end
			end
			if EEL_HasNotStat(EEL_Data_BC,value) then
				additem = false;
			elseif EEL_IsNotSlot(EEL_Data_BC,value) then
				additem = false;
			elseif (EEL_EnchantInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EEL, value);
			end        
			currentenchant = currentenchant + 1;
		end
	end
	
	if (EEL_ShowLevel(EEL_Options["SelectedLevel"],2)) then -- ALL, WotLK
		for value,_ in pairs(EEL_Data_WotLK) do    
			additem = true;
			if EEL_Options["favorites"] then
				if EEL_FAV[value] == nil then
					additem = false;
				end
			end			
			if EEL_HasNotStat(EEL_Data_WotLK,value) then
				additem = false;
			elseif EEL_IsNotSlot(EEL_Data_WotLK,value) then
				additem = false;
			elseif (EEL_EnchantInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EEL, value);
			end        
			currentenchant = currentenchant + 1;
		end
	end
	
	if (EEL_ShowLevel(EEL_Options["SelectedLevel"],3)) then	-- if 4.0.3 + ALL, Cataclysm
		for value,_ in pairs(EEL_Data_Cata) do    
			additem = true;	    
			if EEL_Options["favorites"] then
				if EEL_FAV[value] == nil then
					additem = false;
				end
			end
			if EEL_HasNotStat(EEL_Data_Cata,value) then
				additem = false;
			elseif EEL_IsNotSlot(EEL_Data_Cata,value) then
				additem = false;
			elseif (EEL_EnchantInfo(value) == false) then
				additem = false;
			end
			if additem == true then
				tinsert(EEL, value);
			end        
			currentenchant = currentenchant + 1;
		end      
	end
	
    table.sort(EEL, EEL_SortResults);
    EEL_ShowEnchantList();    
end

function EEL_ScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc)
	if EEL_Options["details"] then
		lineheight = 32;
	end
	FauxScrollFrame_OnVerticalScroll(this, myoffset, lineheight, updatefunc);
end


function EEL_SortResults(a, b)
	local iteminfoA = EEL_EnchantInfo(a);
	local iteminfoB = EEL_EnchantInfo(b);
    local itemNameA, itemLinkA, itemQualityA, itemLevelA = EEL_EnchantInfo(a);
    local itemNameB, itemLinkB, itemQualityB, itemLevelB = EEL_EnchantInfo(b);
	
	if ((iteminfoA ~= nil) and (iteminfoB ~= nil)) then
		if itemLevelA ~= itemLevelB then	-- Sort by item level first
			return itemLevelA > itemLevelB;
		end
		if itemQualityA ~= itemQualityB then	-- Sort by quality
			return itemQualityA > itemQualityB;
		end
		if itemNameA and itemNameB then		-- if same item level, sort by name
			return itemNameA < itemNameB;
		end
	end
	return a < b;						-- sort by id, if no name
end


function EEL_ShowEnchantList()  
	local i, item, itemm, itemn, itemi, itemf, itemr, itemb, itemy;
	local sg = 15;
    MaxItems = eel_tcount(EEL);
	if EEL_Options["details"] then
		lineheight = 32;
		sg = 7;
	else
		lineheight = 16;
		sg = 15;
	end
    FauxScrollFrame_Update(EEL_ScrollBar,MaxItems,sg,lineheight);
    for i=1,sg do
		-- get global button names from xml script
		item = _G["EELItem_"..i];
		itemn = _G["EELItem_"..i.."_Name"];
		itemi = _G["EELItem_"..i.."_Icon"];
		itemf = _G["EELItem_"..i.."_Fav"];
		itemp = _G["EELItem_"..i.."_MarketPrice"];
		iteme = _G["EELItem_"..i.."_Extra"];
				
        lineplusoffset = i + FauxScrollFrame_GetOffset(EEL_ScrollBar);
		if lineplusoffset < MaxItems+1 then
			-- get gem data from local DB or load from server if not available
			local itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats = EEL_EnchantInfo(EEL[lineplusoffset]);
						
			if (itemStats == nil) or (itemStats == "") then
				itemStats = EEL_GetToolTipStats(EEL[lineplusoffset]);
			end
			if itemName then
				if iteme ~= nil then
					-- show detailed info in result list ?
					if EEL_Options["details"] then
						item:SetHeight(30);
						iteme:SetText(itemStats);
						iteme:Show();
					else
						item:SetHeight(15);
						iteme:SetText("");
						iteme:Hide();
					end
				end

				if itemn ~= nil then
					itemn:SetText(ITEM_QUALITY_COLORS[itemRarity].hex..itemName);
				end
				if itemi ~= nil then
					itemi:SetTexture(itemTexture);
				end
				if itemp ~= nil then
					eglprice = EEL_MarketPrice(EEL[lineplusoffset]);
					if eglprice then
						itemp:SetText(eglprice);
					end
				end
				if itemf ~= nil then
					if EEL_FAV[EEL[lineplusoffset]] ~= nil then
						itemf:SetTexture("Interface/GROUPFRAME/UI-Group-MasterLooter");
						itemf:Show();
					else
						itemf:SetTexture("");
						itemf:Hide();
					end
				end
				if item ~= nil then
					item:Show();
				end

			end
		else
			if item ~= nil then
				item:Hide();
			end
		end
    end
	itemm = _G["EEL_ENCHANTCOUNT"];
	if itemm ~= nil then
		local EEL_AUSWAHL = "";
		if EEL_Options["favorites"] then
			EEL_AUSWAHL = " ("..EEL_SHOWFAVORITES..")";
		end
		itemm:SetText(MaxItems.." enchants"..EEL_AUSWAHL..":");
	end
end

-- INITIAL ENCHANT SCAN ------------------------------------------------

function EEL_GetEnchantInfoDelayed(value)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	if itemName then
		local itemStats = EEL_GetToolTipStats(value);
		EELDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats };		
		EEL_ScrollBar_Update();
		if EEL_Options["debug"] then
			out("EEL DEBUG: Added delayed Item ID"..value);
		end
	else
		out("EEL DEBUG: Could not retrieve ItemID "..value.." from server, please report to addon author");
	end
end


function EEL_GetEnchantInfo(value)
	-- REQUEST FROM SERVER (should be ONLY this line)
	itemName, itemLink, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, itemEquipLoc, itemTexture, itemSellPrice = GetItemInfo(value);
	if itemName then
		EEL_Scanned_Enchants = EEL_Scanned_Enchants + 1;
		--if EELDB[value] == nil then -- save GemInfo to DB
			if EEL_Options["debug"] then
				out("EEL Debug: Scanning Tooltip for "..itemName);
			end
			local itemStats = EEL_GetToolTipStats(value);
			EELDB[value] = { itemName, itemLink, itemRarity, itemLevel, itemTexture, itemStats };
		--end
		return true;
	else
		EEL_wait(10,EEL_GetEnchantInfoDelayed,value);
		failed = failed + 1;
		if EEL_Options["debug"] then
			out("EEL DEBUG: Failed to get ItemID "..value);
		end
		return false;
	end
end

function EEL_Scan()
	EELDB = {};	
	EEL_Scanned_Enchants = 0;
	out("EEL: Scanning enchants ...")
	if EEL_Options["cataclysm"] then
		for value,_ in pairs(EEL_Data_Cata) do
			if EEL_Options["debug"] then
				out("EEL Debug: Scanning Cataclysm enchants");
			end
			EEL_GetEnchantInfo(value);
		end
	end
	for value,_ in pairs(EEL_Data_WotLK) do
		if EEL_Options["debug"] then
			out("EEL Debug: Scanning WotLK enchants");
		end
		EEL_GetEnchantInfo(value);
	end
	for value,_ in pairs(EEL_Data_Classic) do
		if EEL_Options["debug"] then
			out("EEL Debug: Scanning Classic enchants");
		end
		EEL_GetEnchantInfo(value);
	end
	for value,_ in pairs(EEL_Data_BC) do
		if EEL_Options["debug"] then
			out("EEL Debug: Scanning BC enchants");
		end
		EEL_GetEnchantInfo(value);
	end
	EEL_FinalizeScan();
end

function EEL_AbortScan()
	eel_scan_state=0;
end

function EEL_ForceScan()
	eel_scan_state = 0;
	EEL_Scan();
end

function EEL_FinalizeScan()
	eel_scan_state=1;
	out("EEL: Scanned "..EEL_Scanned_Enchants.." enchants.");
	if (failed > 0) then
		out("EEL: Missing "..failed.." enchants will be added in a few seconds ...");
		failed = 0;
	end
	EEL_ScrollBar_Update();
end


